// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

/*
 _______ _________ _______  _______  _
(  ____ \\__   __/(  ___  )(  ____ )( )
| (    \/   ) (   | (   ) || (    )|| |
| (_____    | |   | |   | || (____)|| |
(_____  )   | |   | |   | ||  _____)| |
      ) |   | |   | |   | || (      (_)
/\____) |   | |   | (___) || )       _
\_______)   )_(   (_______)|/       (_)

Do not modify this code until you have read the Read Me.txt contained in the root directory of this shaderpack!

*/

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif


#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying highp vec3 world_pos;
varying highp vec3 pos_layar;
varying highp vec3 fragment_pos;
varying highp vec3 look_vector;
#define WEATHERLAYER
varying highp vec3 position;
varying highp vec3 look;
varying highp vec3 w_pos;
varying highp vec3 v_pos;
varying float f_pos;
varying POS3 h_pos;
varying float far_f;
varying vec3 dpos;
varying vec3 screen_pos;
varying highp vec3 lpos;
varying highp vec3 ppos;

#ifdef FOG
varying vec4 fogColor;
#endif

#ifdef GL_FRAGMENT_PRECISION_HIGH
	varying highp vec3 cPos;
#else
	varying mediump vec3 cPos;
#endif
varying POS3 wPos;

#include "uniformShaderConstants.h"
#include "uniformPerFrameConstants.h"
#include "uniformRenderChunkConstants.h"
#include "uniformEntityConstants.h"
#include "util.h"

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

float inrect(vec2 pos, float x1, float y1, float x2, float y2, float focus){
 return min(1.0, max(min(min(pos.x - x1, x2 - pos.x), min(pos.y - y1, y2 - pos.y)), 0.0) / focus);
}

float bongnguoichoi(){
vec3 lookvector=lpos;
lookvector.x*=2.0;
 vec3 pos = lookvector + vec3(0.4, 0.4, 0.4); 
 vec3 dir = vec3(-1.0, (1.25) * 0.31, 0.0);
 float factor = 1.0;
 if (pos.x < 0.2){
  factor = max(0.0, pos.x / 0.4 + 0.5);
 }
pos += dir * pos.x;
 float focus = .04;
 float footwalk = sin((ppos.x - lookvector.x) * 2.0 + (ppos.z - lookvector.z) * 2.0);
float handswalk = sin((ppos.x - lookvector.x) * 2.0 + (ppos.z - lookvector.z)) * .5;
pos.yz -= vec2(.2, .4);
 float body = max(inrect(pos.yz, -1.5 + footwalk*.4, -0.25, 0.75, .1, focus), inrect(pos.yz, -1.5 - footwalk*.4, -.1, 0.75, 0.25, focus));
 float hands = max(inrect(pos.yz, -0.5 + footwalk * .1, -0.5, 0.25, .1, focus), inrect(pos.yz, -0.5 - footwalk * 0.1, -.1, 0.25, 0.5, focus));
 return min(1.0, max(body, hands)) * factor;
}

vec3 U2TM(vec3 color)
{
	float gamma = 1.25;
	float A = 1.25;
	float B = 0.1;
	float C = 0.6;
	float D = 0.7;
	float E = 0.02;
	float F = 0.6;
	float W = 11.2;
	float exposure = 2.;
	color *= exposure;
	color = ((color * (A * color + C * B) + D * E) / (color * (A * color + B) + D * F)) - E / F;
	float white = ((W * (A * W + C * B) + D * E) / (W * (A * W + B) + D * F)) - E / F;
	color /= white;
	color = pow(color, vec3(1. / gamma));
	return color;
}

#ifndef BYPASS_PIXEL_SHADER

vec4 bump_pom(vec4 d, vec2 u, float p){

vec3 pos_pom = vec3(1.0);
float test_pos = (abs(p*0.85));
float max_pos = min(1.0,max(0.0,test_pos));
pos_pom = mix(pos_pom,vec3(0.0,0.0,1.0),max_pos);
pos_pom = mix(pos_pom,vec3(1.0,1.0,0.0),1.0-max_pos);

vec2 set_height = (pos_pom.xy/pos_pom.z) * (d.r *0.001);

vec2 pom = u+(-set_height);
vec4 diffuse_pom =texture2D(TEXTURE_0, pom);
vec4 start_bump = pow(diffuse_pom, vec4(1.0))*0.35;

return start_bump;
}

vec4 wet(vec4 d, vec3 wp){
vec3 pos_kilap = d.rgb*(wp);
vec4 texturescropio = texture2D(TEXTURE_0,pos_kilap.xz);
vec3 lekukan = (vec3(length(abs(texturescropio.xyz)+sin(dot(texturescropio.y,pos_kilap.z)))));
vec3 kilap_buatan= pos_kilap-lekukan.rgb;

vec4 do_weat = d*(length(kilap_buatan));
vec4 start = abs(do_weat)*0.1;
return start; }

vec4 wet_pom(vec4 d, vec3 p, vec3 wp, vec2 u, float r){
vec4 bp = bump_pom(d,u,p.z);
vec4 w = wet(d,wp);
w = d-bp+w;
bp = d*1.2-bp;

return mix(bp,w,r); }

#endif

highp float randff(highp float n){
 return fract(cos(n) * 3782.288);
}

highp float smoothrand(highp float pos){
 highp float start = floor(pos);
 highp float smoothy = smoothstep(0.0, 1.0, fract(pos));
 return mix(randff(start), randff(start + 1.0), smoothy);}

highp float smoothrand2d(highp float horizont, highp float forward){
 highp float start = floor(forward);
 highp float smoothy = smoothstep(0.0, 1.0, fract(forward));
 return mix(smoothrand(horizont + randff(start) * 1700.0), smoothrand(horizont + randff(start + 1.0) * 1700.0), smoothy);
}

highp float CheapWaterMap(highp vec3 pos){
pos.z += TIME * 0.35;
 highp float wave1 = smoothrand2d(pos.z+pos.z+pos.z, pos.x+pos.x+pos.x);
 highp float wave2 = sin(wave1) * 2.5 + 0.5;
float interp = sin(TIME * 2.5 + pos.x+pos.z+pos.x+pos.z) * sin(pos.z) * 0.35;
 return mix(wave1, wave2, interp);
}

highp vec3 WaterNormalMap(highp vec3 pos){
 highp float value = (CheapWaterMap(pos + vec3(0.0, 0.0, 0.1)) - CheapWaterMap(pos));
 return vec3(0.0, 1.0, -value);
}

vec4 getSunColor(highp vec3 pos, float day_flag){
 highp vec3 sun_pos = vec3(0.0, 100.0, -600.0);
 float white = clamp(abs(day_flag - 0.7) / 0.3 + 0.2, 0.0, 1.0);
 vec4 sun_color = vec4(1.0, white, white, 1.0);
 sun_color.a = pow(max(0.0, 1.0 - length(sun_pos.xz - pos.xz) / 160.0), 0.6);
 return sun_color;
}

const vec3 origin_water_normal = vec3(0.0, 1.0, 0.0);

highp float randz(highp vec2 p){ 
	return fract(cos(p.x + p.y * 225.61) * 618.552);
} 

highp float noisez(highp vec2 p) { 
	highp vec2 i = floor(p); 
	highp vec2 f = fract(p);  
	highp vec2 u = f * f * (3.0 - 2.0 * f); 
	return -1.0 + 2.0 * mix(mix(randz(i + vec2(0.0, 0.0)), 
							randz(i + vec2(1.0, 0.0)), u.x), 
							mix(randz(i + vec2(0.0, 1.0)), 
							randz(i + vec2(1.0, 1.0)), u.x), u.y); 
}

highp mat2 rotationmatrix = mat2(0.8, 0.6, - 0.6, 0.8);
highp float noisemap(highp vec2 p, int depth, highp float d){
	highp float t = TIME * 1.0; 
 	d = smoothstep(0.0, 100.0, d);
	p *= 0.3 * (d + 0.2);
	highp float z = 2.0;
	highp float rz = 0.0;
	p -= vec2(0.3, 1.0) * t * 0.05;
	for(int i = 0; i < depth; i++){
		rz += noisez(p) / z * 1.25;
		z *= 2.1;
		p *= 2.15;
		p += t * 0.027 * pow(z, .5);
		p *= rotationmatrix;
	}
	return pow(abs(rz), 2.0 - d);
}

float highstep(float x, float a, float b){
	float k = 1.0 / (b - a);
	return clamp(x * k + (1.0 - b * k), 0.0, 1.0);
}

float cloudlayer(vec2 pos, int depth){
	return noisemap(pos, depth, 1.0) * 4.0;
}

float weathermap(vec2 pos){
	#ifdef WEATHERLAYER
		return highstep(cloudlayer(pos * .075, 2), 0.0, 1.0);
	#else
		return 0.0;
	#endif
}

vec4 cloudcolor(highp vec2 pos, vec3 ambient, vec3 lightcolor){
	highp vec2 dir = vec2(-1.0, -0.5) * 1.82;
	highp float mainlayer = cloudlayer(pos, 6);
	highp float normallayer = cloudlayer(pos - dir, 5);
	highp float weatherlayer = weathermap(pos);
	highp float weatherfactor = 1.0 + weatherlayer * weatherlayer * 100.0;
	highp float mvalue = mainlayer;
	mainlayer *= weatherfactor;
	normallayer *= weatherfactor;

	if(normallayer < 1.0){
		normallayer -= (1.0 - normallayer) * 2.5;
	}
	if(mainlayer < 1.0){
		mainlayer -= (1.0 - mainlayer) * 2.5;
	}

	float layerdelta = (mainlayer - normallayer) / 1.5 / length(dir) / weatherfactor;
	float layervalue = normallayer / 3.0;

	float light = pow(clamp(layerdelta, 0.0, 1.0), 1.0);
	float dark = 1.0 - pow(clamp(layerdelta + 0.1, 0.0, 1.0), 0.9);

float afternoon =pow(max(min(1.0-FOG_COLOR.b*1.2,1.0),0.0),0.5);
float bengi =pow(max(min(1.0-FOG_COLOR.r*1.5,1.0),0.0),0.3);
float udan =(1.0-pow(FOG_CONTROL.y,2.0));
vec3 color_afternoon = FOG_COLOR.rgb* vec3(0.9,0.6,0.3)* ambient+ vec3(0.15,0.0,-0.1);
vec3 color_night = vec3(0.0)*ambient;
vec3 color_rain = FOG_COLOR.rgb*1.4;

	vec3 basecolor = vec3(0.11);
	vec3 brightcolor = vec3(0.65,0.69,0.7) * ambient;

brightcolor = mix(brightcolor, color_afternoon, afternoon);
brightcolor = mix(brightcolor, color_night, bengi);
brightcolor = mix(brightcolor, color_rain, udan);

	vec3 bgcolor = brightcolor * (1.1 - dark * mvalue * (weatherlayer + .1) * 0.125);
	vec3 color = mix(bgcolor, bgcolor + lightcolor, min(light, 1.0));

	return vec4(color, min(1.0, pow(mvalue, .9))); 
}

vec3 getlightcolor(float brightness){
	float sunset = (brightness - 0.75) / 0.3;
	float sunsetval = 1.0 - pow(min(1.0, abs(sunset)), 1.5);
	vec3 sunsetcolor = vec3(0.79, 0.5, 0.45) * 1.8;
	if(sunset > 0.0){
		return mix(vec3(.45,0.25,0.05), sunsetcolor, sunsetval);
	}
	else{
		return mix(vec3(0.25, 0.25, 0.35) * 1.8, sunsetcolor, sunsetval);
	}
}

highp float cos_between_vecs(highp vec3 v1, highp vec3 v2){
 return (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z) / length(v1) / length(v2);
}

highp float rand(highp float n){
 return fract(cos(n) * 3782.288);
}

highp float noise(highp float pos){
 highp float start = floor(pos);
 highp float smoothy = smoothstep(0.0, 1.0, fract(pos));
 return mix(rand(start), rand(start + 1.0), smoothy);}

highp float noise2d(highp float horizont, highp float forward){
 highp float start = floor(forward);
 highp float smoothy = smoothstep(0.0, 1.0, fract(forward));
 return mix(noise(horizont + rand(start) * 1700.0), noise(horizont + rand(start + 1.0) * 1700.0), smoothy);
}

highp float Waves(highp vec3 pos){
pos.z += TIME * 0.35;
 highp float wave1 = noise2d(pos.z+pos.z+pos.z, pos.z+pos.z+pos.x+pos.x);
 highp float wave2 = sin(wave1) * 2.5 + 0.5;
float interp = sin(TIME * 2.5 + pos.z+pos.x+pos.z+pos.x) * sin(pos.z) * 0.35;
 return mix(wave1, wave2, interp);
}

highp vec3 waterMap(highp vec3 pos){
 highp float value = (Waves(pos + vec3(0.0, 0.0, 0.1)) - Waves(pos));
 return vec3(0.0, 1.0, -value);
}

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0);
	return;
#else

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0;
	inColor.b = 1.0;
#endif

float PerformanceCoverage = (0.960);

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < PerformanceCoverage)
		discard;
#endif

                       vec4 inColor = color;


vec4 f_color = FOG_COLOR;
vec4 fog = FOG_COLOR;
vec4 lf_color = mix(fog,vec4(1.0),1.0-fog.b);

#if defined(BLEND)
	diffuse.a *= inColor.a;

#endif

#if !defined(ALWAYS_LIT)
 vec4 Light= texture2D( TEXTURE_1, uv1 );
 diffuse*=Light;
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;

	#endif
	diffuse.rgb *= inColor.rgb;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

vec3 pos_kilap = diffuse.rgb*(world_pos);
vec4 texturespes = texture2D(TEXTURE_0,pos_kilap.xz);
vec3 lekukan = (vec3(length(abs(texturespes.xyz)+sin(dot(texturespes.y,pos_kilap.z)))));
vec4 lekukan2 = texture2D(TEXTURE_0, -vec2(length(lekukan)) );
vec3 kilap_buatan= pos_kilap-lekukan.rgb;

vec4 do_weat = diffuse*(length(kilap_buatan));

float diffuse_pom2 = 1.0;
float do_weat2 = 1.0;
float rain_effect = 1.0;

#ifdef BLEND
do_weat2 = 0.0;
diffuse_pom2 = 0.0;
#endif

if(FOG_CONTROL.x<0.55&&FOG_CONTROL.x>0.1){ 
diffuse += rain_effect*do_weat2*abs(do_weat)*0.12*uv1.y;

vec3 torch = pow(vec3(255, 141, 11) / 255.0 *uv1.x*0.5, vec3(2.2));
diffuse.rgb += torch;

vec3 clr=vec3(1.135,1.105,1.05)*0.5;
diffuse.rgb*=mix(vec3(1.),clr,Light.r);

vec3 clr2=vec3(0.005,0.01,0.02)*0.00003;
diffuse.rgb*=mix(vec3(.0),clr,Light.r);

vec3 br=vec3(0.13);
diffuse.rgb+=mix(vec3(.0),br,Light.r);

#ifdef FANCY
if(color.b>color.r)
{
	diffuse.rgb *=vec3(135./255.,217./255.,211./255.);
}
#endif
 }

POS3 posxz = h_pos;

posxz.x += 0.5*sin(posxz.z*4.0+TIME*5.0)*0.1;
posxz.z += cos(posxz.x*2.0+TIME*0.5)*0.1;
			
float h0 = sin(posxz.x*7.0+posxz.z*0.5+0.1+TIME*2.0)+sin(posxz.z*1.5+2.0+TIME*1.2);
posxz.x = h_pos.x+ 4.5*sin(posxz.z*2.0+TIME*1.2)*0.1;
posxz.z = h_pos.z+8.5*cos(posxz.x*1.0+TIME*2.3)*0.1;
float h1 = cos(posxz.x*6.0+posxz.z/2.5+0.1+TIME*1.5)+cos(posxz.z/1.5+10.2+TIME*2.5);
posxz.x = h_pos.x+ 3.8*cos(posxz.z*4.0+TIME*0.5-far_f)*0.1;
posxz.z = h_pos.z+10.5*cos(posxz.x*3.0+TIME*1.8)*0.1;
float h2 = sin(posxz.x*8.0+posxz.z*0.3+0.2+TIME*1.6)+sin(posxz.z*1.3+8.4+TIME*0.8);
posxz.x = h_pos.x+ 2.3*sin(posxz.z*2.0+TIME*1.5)*0.1;
posxz.z = h_pos.z+7.9*cos(posxz.x*2.0+TIME*2.5)*0.1;
float h3 = cos(posxz.x*7.0+posxz.z/2.5+0.5+TIME*2.5)+cos(posxz.z/2.0+0.5+TIME*1.5);
posxz.x = h_pos.x+ 2.3*cos(posxz.z*3.0+TIME*4.0)*0.1;
posxz.z = h_pos.z+6.2*cos(posxz.x*4.0+TIME*2.0)*0.1;
float h4 = cos(posxz.x*9.0+posxz.z/5.2+0.7+TIME*1.0)+cos(posxz.z/1.2+14.8+TIME*2.0);
posxz.x = h_pos.x+ 2.8*sin(posxz.z*0.5+TIME*2.0)*0.1;
posxz.z = h_pos.z+5.0*cos(posxz.x*3.0+TIME*1.5)*0.1;
float h5 = cos(posxz.x*9.0+posxz.z/3.2+0.7+TIME*1.0)+cos(posxz.z/2.2+6.8+TIME*2.0);
			
float r0 = sin(posxz.x*8.0+posxz.z*0.6+0.1+TIME*2.0)+sin(posxz.z*1.5+2.0+TIME*1.2);
posxz.x = h_pos.x+ 4.5*sin(posxz.z*2.0+TIME*1.0+far_f)*0.1;
posxz.z = h_pos.z+2.5*cos(posxz.x*1.0+TIME*2.5)*0.1;
float r1 = cos(posxz.x*7.0+posxz.z/1.5+0.1+TIME*1.5)+cos(posxz.z/1.5+10.2+TIME*2.5);
posxz.x = h_pos.x+ 3.8*cos(posxz.z*4.0+TIME*1.0-far_f)*0.1;
posxz.z = h_pos.z+0.5*cos(posxz.x*3.0+TIME*1.5)*0.1;
float r2 = sin(posxz.x*9.0+posxz.z*0.4+0.2+TIME*1.6)+sin(posxz.z*1.3+8.4+TIME*0.8);
posxz.x = h_pos.x+ 2.3*sin(posxz.z*2.0+TIME*3.0+far_f)*0.1;
posxz.z = h_pos.z+1.9*cos(posxz.x*2.0+TIME*2.5)*0.1;
float r3 = cos(posxz.x*8.0+posxz.z/1.5+0.5+TIME*2.5)+cos(posxz.z/2.0+0.5+TIME*1.5);
posxz.x = h_pos.x+ 2.3*cos(posxz.z*3.0+TIME*2.5-far_f)*0.1;
posxz.z = h_pos.z+0.2*cos(posxz.x*4.0+TIME*2.0)*0.1;
float r4 = cos(posxz.x*10.0+posxz.z/4.2+0.7+TIME*1.0)+cos(posxz.z/1.2+14.8+TIME*2.0);
posxz.x = h_pos.x+ 2.8*sin(posxz.z*0.5+TIME*3.0+far_f)*0.1;
posxz.z = h_pos.z+1.0*cos(posxz.x*3.0+TIME*1.5)*0.1;
float r5 = cos(posxz.x*8.0+posxz.z/2.2+0.7+TIME*1.0)+cos(posxz.z/2.2+6.8+TIME*2.0);
			
float water_r =abs(r0-r1)+abs(r2-r3)+r4*far_f;
float water_a =(abs(h0+h1)+abs(h2+h3)+h4)*(far_f+0.2);

if(water_a>1.0){
water_a*=0.8*min(abs(far_f+0.2)*4.0,1.0);
}

vec4 un_water_color = vec4(0./255.,233./255.,255./255.,1.0);
vec4 unwater_day_color  = vec4(2.0,1.94,1.78,0.16);

if((f_color.b>.15&&(f_color.b>f_color.g*1.2)&&(f_color.b>=f_color.r*1.8)&&(f_color.g>f_color.r*1.6))||(f_color.g*1.2>=f_color.b)&&(f_color.r*1.3<f_color.g)){

diffuse.rgb *= un_water_color.rgb+abs(unwater_day_color.rgb*vec3(water_a+water_r-abs(water_a*water_r))*0.1);

//diffuse*=1.0+hd+ggg;}

diffuse += rain_effect*do_weat2*abs(do_weat)*0.12*uv1.y;

float udan = (1.0-pow(FOG_CONTROL.y,5.0));

diffuse = wet_pom(diffuse,screen_pos,world_pos,uv0,udan*uv1.y);

}

vec4 cf = color;

float posisi =length(position.xyz);
vec4 fc =FOG_COLOR;
vec4 cc =CURRENT_COLOR;

float set_cloud = uv1.y;
float set_afternoon =pow(max(min(1.0-fc.b*1.2,1.0),0.0),0.5)*uv1.y;
float set_bengi =pow(max(min(1.0-fc.r*1.5,1.0),0.0),0.3)*uv1.y;
float set_udan =(1.0-pow(FOG_CONTROL.y,2.0))*uv1.y;

vec3 color_cave_water = vec3(0./255.,51./255.,102./255.)*0.75;
vec3 color_siang_water = vec3(0.2,0.85,1.0);
vec3 color_afternoon_water = vec3(0.2,0.85,1.0)*0.75;
vec3 color_night_water = vec3(0./255.,51./255.,102./255.);
vec3 color_rain_water = vec3(135./255.,217./255.,211./255.)*fc.rgb;

vec3 color_water = mix(color_cave_water, color_siang_water, set_cloud);
color_water =mix(color_water,color_afternoon_water,set_afternoon);
color_water =mix(color_water,color_night_water,set_bengi);
color_water =mix(color_water,color_rain_water,set_udan); 

float gg = pow(posisi*2.0,1.0);

highp vec3 water = waterMap(position);

 highp float wave_water = acos(abs(cos_between_vecs(look, water)));

vec4 struktur = mix(cf, vec4(color_water,0.6), pow(wave_water/3.0,1.5));

vec4 _color = struktur;
	

		float brightness = sqrt((max(0.0, color.b - 0.31)) / 0.69);

vec4 cloud = cloudcolor((look.zx) + wave_water * 8.0, vec3(brightness), getlightcolor(color.b) * 0.65);
		//cloud = cloudcolor(position.zx * 121.0 / pow(float(6), .3), vec3(brightness), getlightcolor(color.b) * 0.65);

		_color.rgb = mix(_color.rgb, cloud.rgb, cloud.a*set_cloud);
	

highp vec4 pl = mix(struktur*diffuse, _color, pow(wave_water/3.0,1.5)*0.85);

vec4 uvs = texture2D( TEXTURE_1, vec2(uv1.x*0.65,uv1.y));

highp vec3 frag_water_normal = WaterNormalMap(fragment_pos);

highp vec3 water_normal = mix(frag_water_normal, origin_water_normal, min(1.0, length(look_vector.xz) / max(50.0, abs(look_vector.y) * 4.0)));

highp vec3 reflected_dir = normalize(reflect(look_vector, water_normal));

highp vec3 reflected_pos = reflected_dir * 100.0 / reflected_dir.y + look_vector;

vec4 suncolor = getSunColor(reflected_pos * 2.3, uvs.r);

vec3 torch = pow(vec3(255, 141, 11) / 255.0 *uv1.x*1.0, vec3(2.2));
diffuse.rgb += torch;

vec3 clr=vec3(1.135,1.105,1.05);
diffuse.rgb*=mix(vec3(1.),clr,Light.r);

vec3 clr2=vec3(0.005,0.01,0.02)*0.00003;
diffuse.rgb*=mix(vec3(.0),clr,Light.r);

vec3 br=vec3(0.13);
diffuse.rgb+=mix(vec3(.0),br,Light.r);

diffuse.rgb=U2TM(diffuse.rgb);

#ifdef FANCY
if(color.b>color.r)
{
	diffuse.rgb *=vec3(0.2,0.85,1.0)*0.75;
}
#endif

vec2 uvl=uv1;
uvl.y-=bongnguoichoi()*0.5;
  diffuse *= texture2D(TEXTURE_1, uvl);

bool nether = (FOG_COLOR.r > FOG_COLOR.b	&& FOG_COLOR.r < 0.5 && FOG_COLOR.b < 0.2);

if(nether == true){
    vec3 torch = pow(vec3(1.0,0.2,0.05) *uv1.x*1.0, vec3(2.2));
diffuse.rgb += torch;

	vec3 clr=vec3(1.135,1.105,1.05)*0.25;
diffuse.rgb*=mix(vec3(1.),clr,Light.r);

vec3 br=vec3(0.000013);
diffuse.rgb+=mix(vec3(.0),br,Light.r);

	diffuse.rgb=U2TM(diffuse.rgb);
	
float fogf = 0.0;
vec3 shadow_color = vec3(0.768,0.758,0.858)+uv1.x*vec3(0.318,0.318,0.17);

float shdY = 0.8765;
float shdX = color.g;
if(color.a==0.0){
diffuse.rgb *= 1.25;
shdY = 0.8749;
shdX = color.g*1.3;
}

#ifdef FOG
fogf = 0.5;
#endif

if(((uv1.y<shdY-0.000)||(shdX<0.630))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}
if(((uv1.y<shdY-0.001)||(shdX<0.625))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}
if(((uv1.y<shdY-0.002)||(shdX<0.620))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}
if(((uv1.y<shdY-0.003)||(shdX<0.615))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}
	
    float udan = (1.0-pow(FOG_CONTROL.y,5.0));

    diffuse = wet_pom(diffuse,screen_pos,world_pos,uv0,udan*uv1.y);
}

float fogf = 0.0;
vec3 shadow_color = vec3(0.768,0.758,0.858)+uv1.x*vec3(0.318,0.318,0.17);

float shdY = 0.8765;
float shdX = color.g;
if(color.a==0.0){
diffuse.rgb *= 1.25;
shdY = 0.8749;
shdX = color.g*1.3;
}

#ifdef FOG
fogf = 0.5;
#endif

if(((uv1.y<shdY-0.000)||(shdX<0.630))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}
if(((uv1.y<shdY-0.001)||(shdX<0.625))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}
if(((uv1.y<shdY-0.002)||(shdX<0.620))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}
if(((uv1.y<shdY-0.003)||(shdX<0.615))&&fogf==0.0){
diffuse.rgb *= shadow_color;
}

vec4 tex =texture2D( TEXTURE_0, uv0 );
vec4 ender_light = texture2D( TEXTURE_1, vec2(0.0,1.0) );
vec4 ender_lights_color =vec4(0.7,0.25,0.0,0.1);

vec3 mm_light_color =vec3(0.56,0.24,0.0);
vec3 ender_color =vec3(0.5,0.65,0.70);
vec3 mm_light=mm_light_color*pow(uv1.x*1.4,5.0);//pow(uv1.x*2.0,1.0)

float fog_flag1 =f_pos;
float f_shadow_flag =uv1.y;
float ender_fog_flag =FOG_CONTROL.y;
float ender_rain_flag = (1.0-pow(ender_fog_flag,5.0));

if((((f_color.r>f_color.g)&&(f_color.b>f_color.g)&&(f_color.b>f_color.r))&&(f_color.r<0.05&&f_color.b<0.05&&f_color.g<0.05)&&ender_rain_flag<=0.8&&ender_light.r>0.22)&&(f_shadow_flag==0.0&&CURRENT_COLOR.r<=0.3)){

mm_light=vec3(0.4,0.0,0.7)*pow(uv1.x*1.2,5.0);
diffuse.rgb=tex.rgb*ender_color+mm_light;
diffuse.rgb*=inColor.rgb+(vec3(1.0)-inColor.rgb)*pow(uv1.x*1.3,4.0);
fog_flag1 =0.0;

diffuse.rgb += max(water_a+water_r-abs(water_a*water_r),1.5)*0.05*f_color.rgb;

diffuse +=ender_lights_color*max(pow(uv1.x*1.3,5.0),0.0)*max(water_a+water_r-abs(water_a*water_r),0.5)*0.15;

float shd2 = color.r;
if(color.a<=0.1){ shd2 = color.g*1.999; }
if(shd2<0.638){ diffuse.rgb *= 0.7; }

vec3 t=vec3(1.0)*max(0.53,uv1.x);
vec3 shd=vec3(1.25,1.25,1.25)*t;
if(uv1.y<=.87)
{
diffuse.rgb*=shd;
}
if(uv1.y<=.869)
{
diffuse.rgb*=shd;
}
if(uv1.y<=.868)
{
diffuse.rgb*=shd;
}

float shd3 = length(color.rgb) / 3.;
float shd4 = mix( .8, 1., uv1.x );
if(color.a == 0.) shd3 = 1.;
if(color.b + color.b > color.r + color.g) shd3 = 1.;

if( uv1.y < .87 ) {
}else{
if( shd3 < .2 ) diffuse.rgb *= shd4;
if( shd3 < .198 ) diffuse.rgb *= shd4;
if( shd3 < .196 ) diffuse.rgb *= shd4;
}

diffuse.rgb=U2TM(diffuse.rgb);

#ifdef FANCY
if(color.b>color.r)
{
	diffuse.rgb *=vec3(128./255.,0./255.,128./255.);
}

float udan = (1.0-pow(FOG_CONTROL.y,5.0));

diffuse = wet_pom(diffuse,screen_pos,world_pos,uv0,udan*uv1.y);

#endif
}

vec4 light = texture2D( TEXTURE_1, vec2(0.0,1.0) );

float light_flag =abs(light.r+light.g+light.b)/3.0;
float fog_flag =FOG_CONTROL.y;
float night_flag =min((1.0-light_flag)*2.0,1.0);
float dusk_flag =max(1.0-pow(light_flag,10.0),0.0);
float rain_flag = (1.0-pow(fog_flag,5.0));

vec4 sun_light;

vec4 fog_lights_color =vec4(220./255.,219./255.,223./255.,0.1);
vec4 fog_day_color  = vec4(220./255.,219./255.,223./255.,0.01);
vec4 fog_night_color = vec4(220./255.,219./255.,223./255.,0.1)*0.000013;
vec4 fog_sunset_color = vec4(220./255.,219./255.,223./255.,0.05)*0.0013;
vec4 fog_rain_color = vec4(light_flag*0.8);

sun_light =mix(fog_day_color,fog_rain_color,rain_flag);
sun_light = mix(sun_light,fog_sunset_color,dusk_flag*(1.0-rain_flag));
sun_light = mix(sun_light,fog_night_color,night_flag);

float water_flag =0.0;
float f_b_water_flag =0.0;
float green_lea_flag =0.0;
float un_water_flag =0.0;
float dark_sh_flag = uv1.y;
float snow_lea_flag =0.0;

#ifdef FOG

if(f_b_water_flag ==1.0&&(water_flag!=1.0)&&(green_lea_flag!=1.0)){
//diffuse.rgb=vec3(f_color.r*0.78,f_color.g*0.82,f_color.b*0.87);

diffuse.rgb=vec3(f_color.r*0.58,f_color.g*0.72,f_color.b*0.87);
diffuse =mix(diffuse,lf_color,max(water_a+water_r-abs(water_a*water_r),0.1)*0.10);
diffuse +=fog_lights_color*max(pow(uv1.x*1.4,5.0),0.0);
diffuse =mix(diffuse/2.0*sun_light,vec4(0.14,0.24,0.48,1.0),night_flag);
}

#endif

vec3 fog_change = mix(vec3(0.1),f_color.rgb/vec3(1.5,1.48,1.0),dark_sh_flag);

fog_change =mix(fog_change,f_color.rgb,un_water_flag);

if((water_flag!=1.0 && f_b_water_flag !=1.0 )||snow_lea_flag==1.0){

diffuse.rgb =mix(diffuse.rgb,fog_change,min(far_f*0.7,1.0)*abs(f_b_water_flag*dark_sh_flag-1.0)*abs(water_flag*dark_sh_flag-1.0));

}

#ifdef FOG

if(FOG_CONTROL.x>0.55){
if((color.r*1.1<color.b)&&(color.b>color.g*1.05)){ 
diffuse = pl*inColor; 
diffuse += pow(wave_water/3.0,1.5)*0.35; 
diffuse += cloud*uv1.y*0.35; } }

#endif

#ifdef BLEND

diffuse = pl*inColor;
diffuse += pow(wave_water/3.0,1.5)*0.35;
diffuse += cloud*uv1.y*0.35;
diffuse += suncolor * suncolor.a;

#endif

#ifdef FOG
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );
#endif

gl_FragColor = diffuse;

#endif // BYPASS_PIXEL_SHADER
}
